import json
from .signalr import Connection
from .model import api
from .model import constants
from .fc_md_client import MarketDataClient


class MarketDataStream(object):

	def __init__(self, _config, client: MarketDataClient):

		self._config = _config
		self._client = client
		self._handlers = []
		self._error_handlers = []


	def _on_message(self, _message):
		x = json.loads(_message)
		try:
			for _handler in self._handlers:
				_handler(x)
		except:
			raise Exception(constants.RECEIVE_ERROR_MESSAGE)


	def _on_error(self, _error):

		_error = _error

		for _error_handler in self._error_handlers:

			_error_handler(_error)

	def swith_channel(self, channel):
		self.hub_proxy.server.invoke('SwitchChannels', channel)
     
	def start(self, _on_message, _on_error, _selected_channel, *argv):

		self._handlers.append(_on_message)
		self._channel = _selected_channel
		self._error_handlers.append(_on_error)
		headers = {}#utils.default_headers()
		using_jwt = self._client._get_access_token()
		headers['Authorization'] = self._config.auth_type \
					+ constants.ONE_WHITE_SPACE + using_jwt
			
			
		self.connection = Connection(self._config.stream_url + api.SIGNALR, headers)
		
		self.hub_proxy = self.connection.register_hub(api.SIGNALR_HUB_MD)

		self.hub_proxy.client.on(api.SIGNALR_METHOD, self._on_message)

		self.hub_proxy.client.on(api.SIGNALR_ERROR_METHOD, self._on_error)
		
		self.connection.error += _on_error
		self.connection.on_open_callback(lambda: self.hub_proxy.server.invoke('SwitchChannels', _selected_channel))
		self.connection.start()
			


import json
import requests


from .model import constants
from .model import api
from .model import model
from .model import AccessTokenModel
from dataclasses import asdict

class MarketDataClient(object):

	def __init__(self, _config):

		self._config = _config
		self._header = {'Content-Type': 'application/json',
				'Accept': 'application/json'
			}
		self._access_token: AccessTokenModel = None
		self._get_access_token()

	def _make_post_request(self, _url, data: object = None):

		_header = self._header
		payload = json.dumps(asdict(data))
		_api_url = self._config.url + _url
		_response_obj = requests.post(_api_url, headers = _header, data = payload)
		_response = json.loads(_response_obj.content)
		return _response

	def _make_get_request(self, _url: str, req: object):

		_header = self._header
		_header["Authorization"] = "Bearer " + self._get_access_token()
		_api_url = self._config.url + _url
		_response_obj = requests.get(_api_url, params = asdict(req), headers = _header)
		_response = json.loads(_response_obj.content)
		return _response

	def _get_access_token(self):
		if self._access_token == None or self._access_token.is_expired():
			req = model.accessToken(self._config.consumerID, self._config.consumerSecret)
			payload = json.dumps(asdict( req))
			res = requests.post(self._config.url + api.MD_ACCESS_TOKEN , payload, headers=self._header)
			res_obj = model.Response(**(res.json()))
			if res_obj.status == 200:
				ac = model.AccessToken(**res_obj.data)
				self._access_token = AccessTokenModel(ac)
				return self._access_token.get_access_token()
			else:
				raise NameError(res_obj.message)
		return self._access_token.get_access_token()
     
	def access_token(self, _input_data: model.accessToken):
		return self._make_post_request(api.MD_ACCESS_TOKEN, data = _input_data)


	def securities(self, _input_data, _object: model.securities):
		return self._make_get_request(api.MD_SECURITIES, _object)

	def securities_details(self, _input_data,_object: model.securities_details):
		return self._make_get_request(api.MD_SECURITIES_DETAILS, _object)

	def index_components(self, _input_data,_object:model.index_components):
		return self._make_get_request(api.MD_INDEX_COMPONENTS,  _object)

	def index_list(self,_input_data, _object: model.index_list):
		return self._make_get_request(api.MD_INDEX_LIST,   _object)

	def daily_ohlc(self, _input_data, _object: model.daily_ohlc):
		return self._make_get_request(api.MD_DAILY_OHLC,  _object)

	def intraday_ohlc(self, _input_data, _object: model.intraday_ohlc):
		return self._make_get_request(api.MD_INTRADAY_OHLC,  _object)

	def daily_index(self, _input_data, _object: model.daily_index):
		return self._make_get_request(api.MD_DAILY_INDEX,  _object)

	def daily_stock_price(self, _input_data, _object: model.daily_stock_price):
		return self._make_get_request(api.MD_DAILY_STOCK_PRICE,  _object)

	def backtest(self, _input_data, _object: model.backtest):
		return self._make_get_request(api.MD_BACKTEST,  _object)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# signalr_aio/_connection.py
# Stanislav Lazarov


from .events import EventHook
from .hubs import Hub
from .transports import WebsocketTransport
from .helpers import Helpers
from json import dumps, loads
import logging

class Connection(object):
    protocol_version = '1.5'

    def __init__(self, url, headers=None):
        self.url = url
        self.__hubs = {}
        self.__send_counter = -1
        self.hub = None
        self.session = None
        self.headers = headers
        self.received = EventHook()
        self.error = EventHook()
        self.started = False
        self.logger = Helpers.get_logger()
        self.__transport = WebsocketTransport(self.url, self.headers, on_message=self._on_message )
        
        def handle_error(**kwargs):
            error = kwargs["E"] if "E" in kwargs else None
            if error is None:
                return

            self.error.fire(error)

        self.received += handle_error

    def start(self):
        cdata = self._get_conn_data(self.__hubs)
        self.logger.debug("Connection started")
        return self.__transport.start(connectionData=cdata)
    @staticmethod
    def _get_conn_data(hubs):
        conn_data = dumps([{'name': hub for hub in hubs}])
        return conn_data
    
    def on_open_callback(self, callback):
        self.__transport.on_open_callback(callback=callback)
    
    def _on_message(self, data):
        msg = loads(data)
        self.received.fire(**msg)
    
    def on_open(self, data):
        msg = loads(data)
        self.received.fire(**msg)
        
    def register_hub(self, name):
        if name not in self.__hubs:
            if self.started:
                raise RuntimeError(
                    'Cannot create new hub because connection is already started.')
            self.__hubs[name] = Hub(name, self)
            return self.__hubs[name]

    def increment_send_counter(self):
        self.__send_counter += 1
        return self.__send_counter

    def send(self, message):
        self.__transport.send(message)

    def close(self):
        self.__transport.close()



from dataclasses import dataclass

@dataclass
class accessToken:
  consumerID: str = ''
  consumerSecret: str = ''

@dataclass
class securities:
  market: str = ''
  pageIndex: int = 1
  pageSize: int = 100

@dataclass
class securities_details:
  market: str = ''
  symbol: str = ''
  pageIndex: str = ''
  pageSize: str = ''

@dataclass
class index_components:
  indexCode: str = ''
  pageIndex: int = 1
  pageSize: int = 100


@dataclass
class index_list:
  exchange: str = ''
  pageIndex: int = 1
  pageSize: int = 100


@dataclass
class daily_ohlc:
  symbol: str = ''
  fromDate: str = ''
  toDate: str = ''
  pageIndex: int = 1
  pageSize: int = 100
  ascending: bool = True

@dataclass
class intraday_ohlc:
  symbol: str = ''
  fromDate: str = ''
  toDate: str = ''
  pageIndex: int = 1
  pageSize: int = 100
  ascending: bool = True
  resolution: int = 1

@dataclass
class daily_index:
  requestId: str = ''
  indexId: str = ''
  fromDate: str = ''
  toDate: str = ''
  pageIndex: int = 1
  pageSize: int = 100
  orderBy: str = ''
  order: str = ''

@dataclass
class daily_stock_price:
  symbol: str = ''
  fromDate: str = ''
  toDate: str = ''
  pageIndex: int = 1
  pageSize: int = 100
  market: str = ''

@dataclass
class backtest:
  selectedDate: str = ''
  symbol: str = ''

@dataclass
class Response():
    status: int = 500
    message: str = ''
    data: object = None

@dataclass
class AccessToken(object):
    accessToken: str






import threading
import time
from enum import Enum


class ConnectionStateChecker(object):
    def __init__(
            self,
            ping_function,
            keep_alive_interval,
            sleep=1):
        self.sleep = sleep
        self.keep_alive_interval = keep_alive_interval
        self.last_message = time.time()
        self.ping_function = ping_function
        self.running = False
        self._thread = None

    def start(self):
        self.running = True
        self._thread = threading.Thread(target=self.run)
        self._thread.daemon = True
        self._thread.start()

    def run(self):
        while self.running:
            time.sleep(self.sleep)
            time_without_messages = time.time() - self.last_message
            if self.keep_alive_interval < time_without_messages:
                self.ping_function()

    def stop(self):
        self.running = False


class ReconnectionType(Enum):
    raw = 0  # Reconnection with max reconnections and constant sleep time
    interval = 1  # variable sleep time


class ReconnectionHandler(object):
    def __init__(self):
        self.reconnecting = False
        self.attempt_number = 0
        self.last_attempt = time.time()

    def next(self):
        raise NotImplementedError()

    def reset(self):
        self.attempt_number = 0
        self.reconnecting = False


class RawReconnectionHandler(ReconnectionHandler):
    def __init__(self, sleep_time, max_attempts):
        super(RawReconnectionHandler, self).__init__()
        self.sleep_time = sleep_time
        self.max_reconnection_attempts = max_attempts

    def next(self):
        self.reconnecting = True
        if self.max_reconnection_attempts is not None:
            if self.attempt_number <= self.max_reconnection_attempts:
                self.attempt_number += 1
                return self.sleep_time
            else:
                raise ValueError(
                    "Max attemps reached {0}"
                    .format(self.max_reconnection_attempts))
        else:  # Infinite reconnect
            return self.sleep_time


class IntervalReconnectionHandler(ReconnectionHandler):
    def __init__(self, intervals):
        super(IntervalReconnectionHandler, self).__init__()
        self._intervals = intervals

    def next(self):
        self.reconnecting = True
        index = self.attempt_number
        self.attempt_number += 1
        if index >= len(self._intervals):
            raise ValueError(
                "Max intervals reached {0}".format(self._intervals))
        return self._intervals[index]



import re
from setuptools import setup

# python setup.py sdist bdist_wheel
# twine upload dist/*
def _read_long_description():
    try:
        with open("README.md") as fd:
            return fd.read()
    except Exception:
        return None
    


with open("ssi_fc_data/__init__.py", "r") as fd:
    version = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', fd.read(), re.MULTILINE
    ).group(1)


setup(
    name="ssi_fc_data",
    version=version,
    description='FastConnect Data client by Python',
    author="ducdv",
    author_email="ducdv@ssi.com.vn",
    license='MIT',
    platforms=['POSIX', 'Windows'],
    long_description=_read_long_description(),
    long_description_content_type="text/markdown",
    url="https://github.com/SSI-Securities-Corporation/python-fcdata",
    python_requires='>=3.5',
    install_requires=['requests>=2.18.4', 'websocket-client>=1.5.2'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)


#hub protocol
SIGNALR = "v2.0/signalr"
SIGNALR_METHOD = "Broadcast"
SIGNALR_ERROR_METHOD = "Error"
SIGNALR_HUB_MD = "FcMarketDataV2Hub"




#api link
# MD_GEN_TOKEN = 'api/v2/Market/GenerateToken'
MD_ACCESS_TOKEN = 'api/v2/Market/AccessToken'

MD_SECURITIES = 'api/v2/Market/Securities'
MD_SECURITIES_DETAILS = 'api/v2/Market/SecuritiesDetails'
MD_INDEX_COMPONENTS = 'api/v2/Market/IndexComponents'
MD_INDEX_LIST = 'api/v2/Market/IndexList'
MD_DAILY_OHLC = 'api/v2/Market/DailyOhlc'
MD_INTRADAY_OHLC = 'api/v2/Market/IntradayOhlc'
MD_DAILY_INDEX = 'api/v2/Market/DailyIndex'
MD_DAILY_STOCK_PRICE = 'api/v2/Market/DailyStockPrice'
MD_BACKTEST = 'api/v2/Market/BackTest'



AUTH_TOKEN = 'auth'
TWO_FA = '2fa'
OTHER = 'other'
NONE = None
ONE_WHITE_SPACE = ' '
EMPTY_STRING = ''


TOKEN_TYPE_ERROR_MESSAGE = 'Please select token type!'
CONNECTION_LOST_ERROR_MESSAGE = 'Connection lost: Try to reconnect to server!'
RECEIVE_ERROR_MESSAGE = 'Wrong format, message cannot be decoded!'


import base64
from datetime import datetime
import json
from .model import AccessToken

class AccessTokenModel(object):

	def __init__(self, access_token: AccessToken):
    
		tokenSplit = access_token.accessToken.split(".")
		s = tokenSplit[1]
		s += "=" * ((4 - len(s) % 4) % 4) #ugh
		jj = json.loads((base64.b64decode(s)).decode("utf-8"))
		self._token_expire_at = datetime.fromtimestamp(jj['exp'])
		self._access_token = access_token.accessToken

	def get_access_token(self):
		return self._access_token

	def is_expired(self):
		delta = self._token_expire_at - datetime.now()
		if delta.total_seconds() < 3600:
			return True
		return False



import logging
import urllib.parse as parse


class Helpers:
    @staticmethod
    def configure_logger(level=logging.INFO, handler=None):
        logger = Helpers.get_logger()
        if handler is None:
            handler = logging.StreamHandler()
            handler.setFormatter(
                logging.Formatter(
                    '%(asctime)s - %(name)s - %(levelname)s - %(message)s'))
            handler.setLevel(level)
        logger.addHandler(handler)
        logger.setLevel(level)

    @staticmethod
    def get_logger():
        return logging.getLogger("SignalRCoreClient")

    @staticmethod
    def has_querystring(url):
        return "?" in url

    @staticmethod
    def split_querystring(url):
        parts = url.split("?")
        return parts[0], parts[1]

    @staticmethod
    def replace_scheme(
            url,
            root_scheme,
            source,
            secure_source,
            destination,
            secure_destination):
        url_parts = parse.urlsplit(url)

        if root_scheme not in url_parts.scheme:
            if url_parts.scheme == secure_source:
                url_parts = url_parts._replace(scheme=secure_destination)
            if url_parts.scheme == source:
                url_parts = url_parts._replace(scheme=destination)

        return parse.urlunsplit(url_parts)

    @staticmethod
    def websocket_to_http(url):
        return Helpers.replace_scheme(
            url,
            "http",
            "ws",
            "wss",
            "http",
            "https")

    @staticmethod
    def http_to_websocket(url):
        return Helpers.replace_scheme(
            url,
            "ws",
            "http",
            "https",
            "ws",
            "wss"
        )

    @staticmethod
    def get_negotiate_url(url):
        querystring = ""
        if Helpers.has_querystring(url):
            url, querystring = Helpers.split_querystring(url)

        url_parts = parse.urlsplit(Helpers.websocket_to_http(url))

        negotiate_suffix = "negotiate"\
            if url_parts.path.endswith('/')\
            else "/negotiate"

        url_parts = url_parts._replace(path=url_parts.path + negotiate_suffix)

        return parse.urlunsplit(url_parts) \
            if querystring == "" else\
            parse.urlunsplit(url_parts) + "?" + querystring

    @staticmethod
    def encode_connection_id(url, id):
        url_parts = parse.urlsplit(url)
        query_string_parts = parse.parse_qs(url_parts.query)
        query_string_parts["id"] = id

        url_parts = url_parts._replace(
            query=parse.urlencode(
                query_string_parts,
                doseq=True))

        return Helpers.http_to_websocket(parse.urlunsplit(url_parts))
